# encoding: utf-8
from __future__ import absolute_import
import unittest

from utils.bean import BaseBean

__author__ = u'Yonka'


def can_not_init(self, v):
    raise AssertionError(u"can not initiate")


class EnumMetaCls(type):
    def __setattr__(cls, key, value):
        if key != u"__init__":
            value.name = key
        super(EnumMetaCls, cls).__setattr__(key, value)

    def __new__(mcs, name, parents, dct):
        if u"_instances" not in dct:
            dct[u"_instances"] = {}
        return super(EnumMetaCls, mcs).__new__(mcs, name, parents, dct)


class CommonNameValueEnum(BaseBean):
    _instances = {}

    def __init__(self, v):
        self.v = v
        self.name = u""
        self.__class__._instances[v] = self

    def get_value(self):
        return self.v

    def from_value(self, v):
        return self.__class__._instances.get(v)

    def __str__(self):
        return u"Enum(%d)" % self.v

    def to_dict(self):
        return self.v
        # return self.name


class TestEnum(unittest.TestCase):
    def test_common_key_value_enum_class(self):
        class TEnum1(CommonNameValueEnum):
            __metaclass__ = EnumMetaCls

            def __str__(self):
                return u"TEnum1(%d)" % self.v

        TEnum1.A = TEnum1(1)
        TEnum1.B = TEnum1(2)
        TEnum1.__init__ = can_not_init

        res = False
        try:
            TEnum1(3)
        except AssertionError:
            res = True
        assert res
        assert TEnum1.A.name == u"A"
        assert len(TEnum1._instances) == 2

        class TEnum2(CommonNameValueEnum):
            __metaclass__ = EnumMetaCls

            def __str__(self):
                return u"TEnum1(%d)" % self.v

        TEnum2.C = TEnum2(1)
        TEnum2.D = TEnum2(2)
        TEnum2.E = TEnum2(3)
        TEnum2.__init__ = can_not_init

        assert len(TEnum2._instances) == 3
        assert len(CommonNameValueEnum._instances) == 0



# encoding: utf-8

from __future__ import absolute_import
from abc import ABCMeta, abstractmethod

__author__ = u'Yonka'


class AbstractBaseConn(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def read(self):
        pass

    @abstractmethod
    def write(self, protocol_message):
        pass

    @abstractmethod
    def is_open(self):
        pass

    @abstractmethod
    def close(self):
        pass



# encoding: utf-8
import unittest

__author__ = 'Yonka'


class _ListGenericMeta(type):
    def __getitem__(cls, item):
        class _List(List):
            __parameters__ = (item,)

        return _List


class List(list):
    __metaclass__ = _ListGenericMeta


class TestTyping(unittest.TestCase):
    def test_list(self):
        t = List[int]
        assert "__parameters__" in t.__dict__



# -*- coding:utf-8 -*-


from strategy.stragety import SimpleStrategy
from configuration.config import logging


class DemoStrategy(SimpleStrategy):
    def __init__(self, quote_config, trader_config):
        super(DemoStrategy, self).__init__(quote_config, trader_config)

    def on_notify_quote(self, data):
        logging("%s", data)


if __name__ == "__main__":
    from configuration.config import quote_config, trader_config


    simple_strategy = DemoStrategy(quote_config, trader_config)
    if not simple_strategy.init():
        logging.error("Failed to login,exit")
        exit(1)

    # 查询用户的帐号详情
    (result,error)=simple_strategy.req_account_detail()
    if error.errorId!=0:
        logging.error(error.to_dict())
    else:
        for item in result:
            logging.info(item.to_dict())
    # 查询用户的持仓详情
    (result,error)=simple_strategy.req_position_detail()
    if error.errorId!=0:
        logging.error(error.to_json())
    else:
        for item in result:
            logging.info(item.to_json())
    # #查询用户在某个策略上的持仓详情
    # (result,error)=simple_strategy.req_position_statics(trader_config.strategyConnectionInfo.strategyId)
    # if error.errorId!=0:
    #     logging.error(error.to_json())
    # else:
    #     for item in result:
    #         logging.info(item.to_json())
    # #查询用户的委托的订单列表
    # (result,error)= simple_strategy.req_order_list()
    # if error.errorId!=0:
    #     logging.error(error.to_json())
    # else:
    #     for item in result:
    #         logging.info(item.to_json())
    # #查询用户在某个策略上委托的订单列表
    # (result,error) = simple_strategy.req_order_list("Medusa_ni1609_16")
    # if error.errorId!=0:
    #     logging.error(error.to_json())
    # else:
    #     for item in result:
    #         logging.info(item.to_json())    # simple_strategy.req_subscribe("SHFE.rb1610.TICK_FIVE_LEVEL")
    # simple_strategy.req_unsubscribe("SHFE.rb1610.TICK_FIVE_LEVEL")
    # simple_strategy.req_unsubscribe("SHFE.rb1605.TICK_FIVE_LEVEL")
    # order_info = StrategyRequestOrder(
    #         account_id=u"test",
    #         hedge_flag=HedgeFlagType.HEDGE_FLAG_HEDGE,
    #         market=u"SHFE",
    #         product=u"rb",
    #         price=2300,
    #         volume=1,
    #         instrument=u"rb1605",
    #         strategy_id=u"1",
    #         price_type=PriceType.PRTP_FIX,
    #         interface_type=InterfaceType.INTERFACE_FUTURE,
    #         order_operation_type=OperationType.OPT_OPEN_LONG
    # )
    #
    #     # try:
    # data = simple_strategy.req_order(order_info)
    simple_strategy.run()



# encoding: utf-8
from __future__ import with_statement
from __future__ import absolute_import
import logging
import os
import threading
import time
import sys
import unittest

from agent.agent_type import AgentType
from communication.server.protocol import function_code
from data.common.enum import HedgeFlagType
from exception import StrategyErrorCode
from exception.exception import ConnectionBrokenException
from strategy.bean import protoquote
from strategy.bean.bean import StrategyOrderDetail, StrategyOrderError, StrategyError, StrategyAccountDetail, \
    StrategyPositionStatics, \
    StrategyRequestOrder, \
    StrategyConnectionInfo, StrategyConfiguration, StrategyQuoteDataRequest
from strategy.bean.databean import KlineBean, TickBean
from strategy.bean.enum import PriceType, InterfaceType, OperationType
from strategy.bean.datatype import *
from strategy.client.client import StrategyClient
from strategy.interface import AbstractStrategy, AbstractNotifyMessageHandler
from strategy.client.interface import AbstractAsyncMessageHandler
from strategy.request import RequestInfo, ResponseInfo
from utils import bean as utils_bean

__author__ = u'yonka'


class BaseStrategy(AbstractStrategy, AbstractNotifyMessageHandler, AbstractAsyncMessageHandler):
    DEFAULT_REQUEST_TIMEOUT = 20000

    def __init__(self, strategy_configuration):
        self.connClient = StrategyClient(
            strategy_configuration.host,
            strategy_configuration.port,
            strategy_configuration.readTimeout,
            strategy_configuration.connTimeout,
            self
        )
        self.requestId = int(time.time() * 1000)
        self.externalAsyncMessageHandler = None
        self.externalNotifyMessageHandler = self
        self.strategyConnectionInfo = strategy_configuration.strategyConnectionInfo
        self.autoReconnect = False
        self.login = False
        self.agentType = strategy_configuration.usageType
        self.reconnectLock = threading.Lock()
        self.finish = False

    def init(self):
        if self.connClient.connect():
            if self.default_req_login():
                self.login = True
            else:
                logging.warning(u"self.default_req_login() failed")
        else:
            logging.warning(u"self.connClient.connect() failed")
        return self.login

    def finish(self):
        self.finish = True

    def run(self, timeout=None):
        runtime = 0
        while (not self.finish):
            time.sleep(1)
            runtime += 1
            if (timeout is not None):
                if runtime >= timeout:
                    break

    def reconnect(self):
        with self.reconnectLock:
            if self.connClient.is_open() and self.login:
                return
            self.login = False
            self.connClient.close()
            if not self.init():
                raise RuntimeError(u"auth failed, pls check connection info")

    def _get_and_inc_req_id(self):
        self.requestId += 1
        return self.requestId

    def _build_request_info(self, timeout, fc):
        request_info = RequestInfo()
        request_info.fc = fc
        request_info.dataType = self.agentType
        request_info.timeout = timeout
        request_info.reqId = self._get_and_inc_req_id()
        return request_info

    @classmethod
    def _convert_2_strategy_error(cls, message):
        return StrategyError().from_json(message.body)

    @classmethod
    def _new_500_error(cls):
        return StrategyError(error_id=500, error_msg=u"system error when call reqQuoteData")

    @classmethod
    def _new_succeed_error(cls):
        return StrategyError(error_id=0, error_msg=u"ok")

    def _handle_request_exception(self, e, safe=True):
        try:
            if isinstance(e, ConnectionBrokenException):
                if self.autoReconnect:
                    self.reconnect()
                else:
                    raise e
            else:
                # XXX java code todo
                pass
        except Exception as e1:
            if safe:
                logging.exception("_handle_request_exception met error: %s", e)
            else:
                raise e1

    def default_req_login(self):
        return StrategyErrorCode.is_success(self.req_login(self.strategyConnectionInfo).errorId)

    def req_subscribe(self, symbol):
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_SUBSCRIBE)
            request_info.data = symbol
            response_info = self.connClient.sync_rpc(request_info)
            return self._convert_2_strategy_error(response_info.protocolMessage)
        except StrategyError as se:
            return se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            return self._new_500_error()

    def req_order_list(self, strategy_id):
        result = None
        err = self._new_succeed_error()
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_ORDERLIST)
            request_info.data = strategy_id
            response_info = self.connClient.sync_rpc(request_info)
            data = response_info.data
            if response_info.is_success():
                result = utils_bean.json_load_bean_list(data, StrategyOrderDetail)
            else:
                err = StrategyError().from_json(data)
        except StrategyError as se:
            logging.exception("%s", se)
            err = se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            err = self._new_500_error()
        return result, err

    def req_account_detail(self, strategy_id):
        result = None
        err = self._new_succeed_error()
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_ACCOUNTETAIL)
            request_info.data = strategy_id
            response_info = self.connClient.sync_rpc(request_info)
            data = response_info.data
            if response_info.is_success():
                result = utils_bean.json_load_bean_list(data, StrategyAccountDetail)
            else:
                err = StrategyError().from_json(data)
        except StrategyError as se:
            logging.exception("%s", se)
            err = se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            err = self._new_500_error()
        return result, err

    def req_position_statics(self, strategy_id):
        result = None
        err = self._new_succeed_error()
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_POTITIONDSTATICS)
            request_info.data = strategy_id
            response_info = self.connClient.sync_rpc(request_info)
            data = response_info.data
            if response_info.is_success():
                result = utils_bean.json_load_bean_list(data, StrategyPositionStatics)
            else:
                err = StrategyError().from_json(data)
        except StrategyError as se:
            logging.exception("%s", se)
            err = se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            err = self._new_500_error()
        return result, err

    def req_cancel_order(self, cancel_order_info):
        return None

    def req_stock_tick(self, symbol=None, begin_date=-1, begin_time=-1, end_date=-1, end_time=-1):
        req_data = StrategyQuoteDataRequest(market="", symbol=symbol, dateType=TICK_STOCK, beginDate=begin_date,
                                            endDate=end_date, beginTime=begin_time, endTime=end_time)
        return self.req_quota_data(req_data)

    def req_future_tick(self, symbol=None, begin_date=-1, begin_time=-1, end_date=-1, end_time=-1, auto_fill=0):
        req_data = StrategyQuoteDataRequest(market="", symbol=symbol, dateType=TICK_FUTURE,
                                            beginDate=begin_date, endDate=end_date, beginTime=begin_time,
                                            endTime=end_time,
                                            autoFill=auto_fill)
        return self.req_quota_data(req_data)

    def req_kline(self, symbol=None, data_type=None, cyc_def=1, begin_date=-1, begin_time=-1, end_date=-1, end_ime=-1,
                  auto_fill=0, cq_flag=0, cq_date=-1, qj_flag=0):
        req_data = StrategyQuoteDataRequest(market="", symbol=symbol, dataType=data_type, cycCount=cyc_def,
                                            beginDate=begin_date, endDate=end_date, beginTime=begin_time,
                                            endTime=end_ime,
                                            autoFill=auto_fill, cqFlag=cq_flag, cqDate=cq_date, qjFlag=qj_flag)
        return self.req_quota_data(req_data)

    def req_quota_data(self, quote_data_category):
        result = None
        err = self._new_succeed_error()
        logging.info("Request data:%s", quote_data_category.to_json())
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_DATA)
            request_info.data = quote_data_category.to_json()
            response_info = self.connClient.sync_rpc(request_info)
            data = response_info.protocolMessage.byteBody
            data_type = quote_data_category.dataType
            if response_info.is_success():
                if data_type == TICK_STOCK or data_type == TICK_STOCK:
                    result = utils_bean.history_data_proto_2_bean_list(data, protoquote.TickBean, TickBean)
                elif data_type == TICK_FUTURE:
                    result = utils_bean.history_data_proto_2_bean_list(data, protoquote.TickBean, TickBean)
                elif SAMPLE_DATA_END > data_type > SAMPLE_DATA_BEGIN:
                    result = utils_bean.history_data_proto_2_bean_list(data, protoquote.KLineBean, KlineBean)
            else:
                err = StrategyError().from_json(data)
        except StrategyError as se:
            logging.exception("%s", se)
            err = se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            err = self._new_500_error()
        return result, err

    # def req_position_detail(self, strategy_id: str) -> (List[StrategyPositionDetail], StrategyError):
    # result = None
    # err = None
    # try:
    #         request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_POTITIONDETAIL)
    #         request_info.data = strategy_id
    #         response_info = self.connClient.sync_rpc(request_info)
    #         data = response_info.data
    #         if response_info.is_success():
    #             result = utils_bean.json_load_bean_list(data, StrategyPositionDetail)
    #         else:
    #             err = StrategyError().from_json(data)
    #     except StrategyError as se:
    #         logging.exception("%s", se)
    #         err = se
    #     except Exception as e:
    #         logging.exception("%s", e)
    #         self._handle_request_exception(e)
    #         err = self._new_500_error()
    #     return result, err
    def req_future_order(self, symbol=None, operation=None, price_type=None, price=None, volume=None, strategy_id=None,
                         hedge_type=HedgeFlagType.HEDGE_FLAG_HEDGE):
        if strategy_id is None:
            strategy_id = self.strategyConnectionInfo.strategyId
        return self.req_order(symbol, operation, price_type, price, volume, strategy_id, hedge_type,
                              InterfaceType.INTERFACE_FUTURE)

    def req_stock_order(self, symbol, operation, price_type, price, volume, strategy_id):
        if strategy_id is None:
            strategy_id = self.strategyConnectionInfo.strategyId
        return self.req_order(symbol, operation, price_type, price, volume, strategy_id, InterfaceType.INTERFACE_STOCK)

    def req_order(self, symbol, operation, price_type, price, volume, strategy_id, interface_type):
        order = StrategyRequestOrder(price=price, price_type=price_type, instrument=symbol,
                                     order_operation_type=operation, interface_type=interface_type,
                                     strategy_id=strategy_id, volume=volume)
        return self.req_order(order)

    def req_order(self, order_info):
        result = None
        err = self._new_succeed_error()
        logging.debug("Requested order info:%s", order_info.to_json())
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_ORDER)
            order_info.strategyClientID = os.getpid()  # XXX 是不是这个？
            request_info.data = order_info.to_json()
            response_info = self.connClient.sync_rpc(request_info)
            data = response_info.data
            if response_info.is_success():
                result = data
                logging.info("response order: ", data)
            else:
                err = StrategyError().from_json(data)
        except StrategyError as se:
            logging.exception("%s", se)
            err = se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
        return result, err

    def req_unsubscribe(self, symbol):
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.REQ_ST_UNSUBSCRIBE)
            request_info.data = symbol
            response_info = self.connClient.sync_rpc(request_info)
            return self._convert_2_strategy_error(response_info.protocolMessage)
        except StrategyError as se:
            logging.exception("%s", se)
            return se
        except Exception as e:
            logging.exception("%s", e)
            self._handle_request_exception(e)
            return self._new_500_error()

    def req_login(self, conn_info):
        try:
            request_info = self._build_request_info(self.DEFAULT_REQUEST_TIMEOUT, function_code.STRATEGYLOGIN)
            conn_info.strategyClientID = os.getpid()  # XXX 确定是不是要这个？
            # XXX 把时间类型转为 "yyyy-MM-dd HH:mm:ss" 格式，但StrategyConnectionInfo没有时间类型啊，囧...
            data = conn_info.to_json()
            request_info.data = data
            response_info = self.connClient.sync_rpc(request_info)
            err = self._convert_2_strategy_error(response_info.protocolMessage)
            if err.errorId == 0:
                logging.info(u"请求连接成功，err is %s", err.to_json())
            else:
                logging.exception(u"请求连接失败，err is %s", err.to_json())
                sys.exit(1)
            return err
        except StrategyError as se:
            logging.exception("%s", se)
            return se
        except Exception as e:
            logging.exception("%s", e)
            return self._new_500_error()

    def on_notify_trade_detail(self, trade_detail):
        pass

    def on_notify_order_error(self, order_error):
        pass

    def on_notify_order_detail(self, order_detail):
        pass

    def on_notify_quote(self, dataType, data):
        pass

    def on_notify_cancel_error(self, order_cancel_error):
        pass

    def handle_async_message(self, async_message):
        def notify_st_subscribe_handler(self, async_message):
            try:
                response_info = ResponseInfo(protocol_message=async_message)
                data = None
                if response_info.dataType == TICK_FUTURE or response_info.dataType == TICK_STOCK:
                    data = utils_bean.proto_to_bean(response_info.data, protoquote.TickBean, TickBean)
                elif SAMPLE_DATA_BEGIN < response_info.dataType < SAMPLE_DATA_END:
                    data = utils_bean.proto_to_bean(response_info.data, protoquote.KLineBean, KlineBean)
                elif KLINE < response_info.dataType < KLINE_END:
                    data = utils_bean.proto_to_bean(response_info.data, protoquote.KLineBean, KlineBean)
                if data:
                    self.externalNotifyMessageHandler.on_notify_quote(response_info.dataType, data)
            except Exception, e:
                logging.exception(u"%s", e)

        def notify_st_orderdetail_handler(self, async_message):
            try:
                logging.debug(u"return orderdetail: %s", async_message.body)
                strategy_order_detail = StrategyOrderDetail().from_json(async_message.body)
                self.externalNotifyMessageHandler.on_notify_order_detail(strategy_order_detail)
            except Exception, e:
                logging.exception(u"%s", e)

        def notify_st_ordererror_handler(self, async_message):
            try:
                strategy_order_error = StrategyOrderError().from_json(async_message.body)
                self.externalNotifyMessageHandler.on_notify_order_error(strategy_order_error)
            except Exception, e:
                logging.exception(u"%s", e)

        def notify_st_unsubscribe_handler(self, async_message):
            try:
                logging.info(u"成功取消订阅： %s", async_message.body)
            except Exception:
                # XXX java代码里todo，处理exception
                pass

        _base_handlers = {
            function_code.NOTIFY_ST_SUBSCRIBE: notify_st_subscribe_handler,
            function_code.NOTIFY_ST_ORDERDETAIL: notify_st_orderdetail_handler,
            function_code.NOTIFY_ST_ORDERERROR: notify_st_ordererror_handler,
            function_code.NOTIFY_ST_UNSUBSCRIBE: notify_st_unsubscribe_handler
        }

        def _base_handle_async_message(self, async_message):
            handler = _base_handlers.get(async_message.fc)
            if handler is not None:
                handler(self, async_message)

        if self.externalAsyncMessageHandler is not None:
            self.externalAsyncMessageHandler.handle_async_message(self, async_message)
        else:
            _base_handle_async_message(self, async_message)


class SimpleStrategy(AbstractStrategy, AbstractNotifyMessageHandler):
    def __init__(
            self,
            quote_config,
            trader_config,
    ):
        self.quoteStrategy = BaseStrategy(quote_config)
        self.traderStrategy = BaseStrategy(trader_config)

    def init(self):
        success = False
        for name, strategy in [(u"trader", self.traderStrategy), (u"quote", self.quoteStrategy)]:
            if strategy is not None:
                try:
                    success = strategy.init()
                    if success:
                        logging.info(u"%s connection login succeed", name)
                    else:
                        logging.exception(u"%s connection failed.....", name)
                except Exception, e:
                    logging.exception(u"login failed, e is %s", e)
                    return False
        return success

    def req_account_detail(self, strategy_id=None):
        return self.traderStrategy.req_account_detail(strategy_id)

    def req_stock_tick(self, symbol=None, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1):
        return self.quoteStrategy.req_stock_tick(symbol, beginDate, beginTime, endDate, endTime)

    def req_future_tick(self, symbol=None, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1, autoFill=0):
        return self.quoteStrategy.req_future_tick(symbol, beginDate, beginTime, endDate, endTime)

    def req_kline(self, symbol=None, dataType=None, cycDef=1, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1,
                  autoFill=0, cqFlag=0, cqDate=-1, qjFlag=0):
        return self.quoteStrategy.req_kline(symbol, dataType, cycDef, beginDate, beginTime, endDate, endTime, autoFill,
                                            cqFlag, cqDate, qjFlag)

    def req_order_list(self, strategy_id=None):
        return self.traderStrategy.req_order_list(strategy_id)

    def req_order(self, order_info):
        return self.traderStrategy.req_order(order_info)

    def req_future_order(self, symbol=None, operation=None, price_type=None, price=None, volume=None, strategy_id=None,
                         hedge_type=HedgeFlagType.HEDGE_FLAG_HEDGE):
        return self.traderStrategy.req_order(symbol, operation, price_type, price, volume, strategy_id, hedge_type)

    def req_stock_order(self, symbol, operation, price_type, price, volume, strategy_id):
        return self.traderStrategy.req_order(symbol, operation, price_type, price, volume, strategy_id,
                                             InterfaceType.INTERFACE_STOCK)

    def req_order(self, symbol, operation, price_type, price, volume, strategy_id, interface_type):
        return self.traderStrategy.req_order(symbol, operation, price_type, price, volume, strategy_id, interface_type)

    def req_login(self, conn_info):
        self.traderStrategy.req_login(conn_info)
        err = self.quoteStrategy.req_login(conn_info)
        logging.info("req_login, err is ", err.to_json())
        return err

    def req_subscribe(self, symbol):
        return self.quoteStrategy.req_subscribe(symbol)

    def req_position_statics(self, strategy_id=None):
        return self.traderStrategy.req_position_statics(strategy_id)

    def req_cancel_order(self, cancel_order_info):
        return self.traderStrategy.req_cancel_order(cancel_order_info)

    def req_position_detail(self, strategy_id=None):
        return self.traderStrategy.req_position_detail(strategy_id)

    def req_unsubscribe(self, symbol):
        return self.quoteStrategy.req_unsubscribe(symbol)

    def on_notify_order_error(self, order_error):
        logging.exception("order_error: ", order_error.to_json())

    def on_notify_trade_detail(self, trade_detail):
        print(trade_detail)

    def on_notify_order_detail(self, order_detail):
        logging.info("order detail: %s", order_detail)

    def on_notify_cancel_error(self, order_cancel_error):
        print(order_cancel_error)

    def on_notify_quote(self, dataType, data):
        pass

    def run(self):
        while (True):
            time.sleep(1000)


class TestStrategy(unittest.TestCase):
    # trader_port = 8280
    trader_port = 8080
    # quote_port = 8380
    quote_port = 8180
    trader_addr = u"101.200.228.73"
    quote_addr = u"101.200.228.73"

    @unittest.skip(u"")
    def test_quote_strategy(self):
        logging.basicConfig(format=u'%(asctime)s %(message)s', datefmt=u'%m/%d/%Y %I:%M:%S %p', level=logging.INFO)
        quote_conn_info = StrategyConnectionInfo(
            account_id=u"016860",
            strategy_id=u"1",
            tcfs=AgentType.QUOTE
        )
        quote_config = StrategyConfiguration(
            self.quote_addr, self.quote_port, strategy_connection_info=quote_conn_info)

        quote_strategy = BaseStrategy(quote_config)
        quote_strategy.init()

        time.sleep(1000)

    def test_simple_strategy(self):
        logging.basicConfig(format=u'%(asctime)s %(message)s', datefmt=u'%m/%d/%Y %I:%M:%S %p', level=logging.INFO)
        trader_conn_info = StrategyConnectionInfo(
            ip=u"101.200.228.73",
            port=unicode(self.trader_port),
            account_id=u"016860",
            password=u"96e79218965eb72c92a549dd5a330112",
            strategy_id=u"1",
            tcfs=AgentType.TRADE
        )
        trader_config = StrategyConfiguration(
            self.trader_addr, self.trader_port, strategy_connection_info=trader_conn_info)
        quote_conn_info = StrategyConnectionInfo(
            account_id=u"016860",
            strategy_id=u"1",
            tcfs=AgentType.QUOTE
        )
        quote_config = StrategyConfiguration(
            self.quote_addr, self.quote_port, strategy_connection_info=quote_conn_info)

        simple_strategy = SimpleStrategy(quote_config, trader_config)
        simple_strategy.init()

        order_info = StrategyRequestOrder(
            account_id=u"016860",
            hedge_flag=HedgeFlagType.HEDGE_FLAG_HEDGE,
            market=u"SHFE",
            product=u"rb",
            price=2100,
            volume=1,
            instrument=u"rb1605",
            strategy_id=u"1",
            price_type=PriceType.PRTP_FIX,
            interface_type=InterfaceType.INTERFACE_FUTURE,
            order_operation_type=OperationType.OPT_CLOSE_SHORT_TODAY
        )

        # try:
        # data = simple_strategy.req_order(order_info)
        # logging.info("data is: %s", data)
        # except Exception as e:
        # logging.exception("e: %s", e)
        # raise e

        try:
            account_detail, err = simple_strategy.req_account_detail(u"1")
            logging.info(u"account_detail: %s", account_detail.to_json() if account_detail else account_detail)
        except Exception, e:
            logging.exception(u"e: %s", e)
            raise e

        # try:
        # position_detail, err = simple_strategy.req_position_detail(u"1")
        #     logging.info(u"position_detail: %s", position_detail.to_json() if position_detail else position_detail)
        # except Exception, e:
        #     logging.exception(u"e: %s", e)
        #     raise e

        try:
            position_statics, err = simple_strategy.req_position_statics(u"1")
            logging.info(u"position_detail: %s", position_statics.to_json() if position_statics else position_statics)
        except Exception, e:
            logging.exception(u"e: %s", e)
            raise e

        time.sleep(1000)



# encoding: utf-8

from __future__ import absolute_import
from abc import ABCMeta, abstractmethod

__author__ = u'Yonka'


class AbstractAsyncMessageHandler(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def handle_async_message(self, async_message):
        pass


class AbstractStrategyClient(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def sync_rpc(self, request_info):
        pass

    @abstractmethod
    def async_rpc(self, request_info):
        pass

    @abstractmethod
    def is_open(self):
        pass

    @abstractmethod
    def close(self):
        return

    @abstractmethod
    def connect(self):
        pass



# encoding: utf-8
from __future__ import absolute_import
from socket import socket

__author__ = u'Yonka'

from communication.client.connection.connection import BaseConn


def new_conn(host, port, read_timeout, conn_timeout):
    s = socket()
    # as python only provides one timeout option which will be used both in connect and read
    if conn_timeout > 0:
        s.settimeout(conn_timeout)  # 以主机名访问时connect需要花费一些时间，与non-blocking冲突（0 timeout）
    s.connect((host, port))
    if read_timeout > 0:
        s.settimeout(read_timeout)  # 0表示非阻塞，模型差异较大
    return BaseConn(s)



#!/usr/bin/env python
# _*_coding:utf-8_*_
# vim : set expandtab ts=4 sw=4 sts=4 tw=100 :

import logging
from logging.handlers import TimedRotatingFileHandler


def configure_log(filename):
    # 日志打印格式
    log_fmt = '%(asctime)s\t%(filename)s,line %(lineno)s\t%(levelname)s: %(message)s'
    formatter = logging.Formatter(log_fmt)
    #创建TimedRotatingFileHandler对象
    log_file_handler = TimedRotatingFileHandler(filename=filename, when="D", interval=1, backupCount=12)
    log_file_handler.setFormatter(formatter)
    logging.basicConfig(level=logging.DEBUG,
                        datefmt='%m/%d/%Y %I:%M:%S %p',
                        filemode='a')
    log = logging.getLogger()
    # 定义一个Handler打印INFO及以上级别的日志到sys.stderr
    console = logging.StreamHandler()
    console.setLevel(logging.DEBUG)
    # 设置日志打印格式
    formatter = logging.Formatter(log_fmt)
    console.setFormatter(formatter)
    # 将定义好的console日志handler添加到root logger
    log.addHandler(console)
    log.addHandler(log_file_handler)
    return log


if __name__ == "__main__":
    configure_log()


# -*- coding:utf-8 -*-
from strategy.bean.datatype import *

from strategy.stragety import BaseStrategy
from configuration.config import logging


class DataExample(BaseStrategy):
    def __init__(self, quote_config):
        super(DataExample, self).__init__(quote_config)

    def on_notify_quote(self, dataType, data):
        if dataType == TICK_STOCK or dataType == TICK_FUTURE:
            logging.info("Received stock data:%s", data.to_dict())
        elif KLINE < dataType < KLINE_END:
            logging.info("Received kline data:%s", data.to_dict())
        else:
            logging.warn("Unexpected data type:%d,content:%s", dataType, data.to_dict())


if __name__ == "__main__":
    from configuration.config import quote_config

    print quote_config.to_json()
    dataExample = DataExample(quote_config)

    if not dataExample.init():
        logging.error("Failed to login,exit")
        exit(1)
    # #######订阅上期所行情
    symbol="%d.%s.%s"%(TICK_LEVEL_TWO_FUTURE,"SHFE","rb1610")
    ret=dataExample.req_subscribe(symbol)
    if ret.errorId!=0:
        logging.error("Failed to subscribe %s because of %s",symbol,ret.to_json())
    # #######订阅中金所行情
    # symbol="%d.%s.%s"%(TICK_LEVEL_TWO_FUTURE,"CFFEX","IC1610")
    # ret=dataExample.req_subscribe(symbol)
    # if ret.errorId!=0:
    # logging.error("Failed to subscribe %s because of %s",symbol,ret.to_json())
    # #######订阅大商所行情
    # symbol="%d.%s.%s"%(TICK_LEVEL_TWO_FUTURE,"DCE","i1609")
    # ret=dataExample.req_subscribe(symbol)
    # if ret.errorId!=0:
    #     logging.error("Failed to subscribe %s because of %s",symbol,ret.to_json())
    # #######订阅郑商所行情
    # symbol="%d.%s.%s"%(TICK_LEVEL_TWO_FUTURE,"CZCE","CF609")
    # ret=dataExample.req_subscribe(symbol)
    # if ret.errorId!=0:
    #     logging.error("Failed to subscribe %s because of %s",symbol,ret.to_json())
    #######订阅上交所行情
    symbol="%d.%s.%s"%(TICK_LEVEL_TWO_STOCK,"SHSE","600600")
    ret=dataExample.req_subscribe(symbol)
    if ret.errorId!=0:
        logging.error("Failed to subscribe %s because of %s",symbol,ret.to_json())
    #######订阅深交所行情
    symbol = "%d.%s.%s" % (KLINE_ONE_MINUTE, "SHFE", "RB1610")
    ret = dataExample.req_subscribe(symbol)
    if ret.errorId != 0:
        logging.error("Failed to subscribe %s because of %s", symbol, ret.to_json())
    symbol = "%d.%s.%s" % (KLINE_ONE_MINUTE, "SHSE", "600600")
    ret = dataExample.req_subscribe(symbol)
    if ret.errorId != 0:
        logging.error("Failed to subscribe %s because of %s", symbol, ret.to_json())
    (data, error) = dataExample.req_kline("SHSE.600895", data_type=SAMPLE_DATA_MINUTE, cyc_def=5, begin_date=20160506,
                                          begin_time=90000000,end_date=20160506,end_ime=94000000)
    for item in data:
        print item.to_json()
    dataExample.run()



# -*- coding:utf-8 -*-
from strategy.stragety import SimpleStrategy
from configuration.config import logging

__author__ = 'patronfeng'


class FutureArbitrate(SimpleStrategy):
    def __init__(self, quote_config, trader_config):
        super(FutureArbitrate, self).__init__(quote_config, trader_config)
        self.symbol_near = ""
        self.symbol_remote = ""
        self.buy_ratio = 1.5
        self.close_profit = 3.5
        self.stop_loss = 2.0
        self.target_volume = 1
        self.direction = 0
        self.symbol_near_price = None
        self.symbol_remote_price = None

    def init(self):
        # 初始化历史数据或者当前持仓情况
        ret = self.req_subscribe(self.symbol_near)
        if ret.errorId != 0:
            logging.error("Failed to subscribe %s because of %s; exit", self.symbol_near, ret.to_json())
            exit(1)
        ret = self.req_subscribe(self.symbol_remote)
        if ret.errorId != 0:
            logging.error("Failed to subscribe %s because of %s", self.symbol_remote, ret.to_json())
            exit(1)

    def on_notify_quote(self, dataType, data):
        pass


from __future__ import absolute_import
from abc import ABCMeta, abstractmethod
import unittest

__author__ = u'Yonka'


class AbstractStrategy(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def req_login(self, conn_info):
        pass

    @abstractmethod
    def req_subscribe(self, symbol):
        pass

    @abstractmethod
    def req_unsubscribe(self, symbol):
        pass

    @abstractmethod
    def req_stock_tick(self, symbol=None, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1):
        pass

    @abstractmethod
    def req_future_tick(self, symbol=None, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1, autoFill=0):
        pass

    @abstractmethod
    def req_kline(self, symbol=None, dateType=None, cycDef=1, beginDate=-1, beginTime=-1, endDate=-1, endTime=-1,
                  autoFill=0, cqFlag=0, cqDate=-1, qjFlag=0):
        pass

    @abstractmethod
    def req_account_detail(self, strategy_id):
        pass

    @abstractmethod
    def req_position_statics(self, strategy_id):
        pass

    @abstractmethod
    def req_order_list(self, strategy_id):
        pass

    @abstractmethod
    def req_order(self, order_info):
        pass

    @abstractmethod
    def req_future_order(self, symbol, operation, price_type, price, volume, strategy_id, hedge_type):
        pass

    @abstractmethod
    def req_stock_order(self, symbol, operation, price_type, price, volume, strategy_id):
        pass

    @abstractmethod
    def req_order(self, symbol, operation, price_type, price, volume, strategy_id, hedge_type, interface_type):
        pass

    @abstractmethod
    def req_cancel_order(self, order_id):
        pass


class AbstractNotifyMessageHandler(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def on_notify_order_detail(self, order_detail):
        pass

    @abstractmethod
    def on_notify_trade_detail(self, trade_detail):
        pass

    @abstractmethod
    def on_notify_order_error(self, order_error):
        pass

    @abstractmethod
    def on_notify_cancel_error(self, order_cancel_error):
        pass

    @abstractmethod
    def on_notify_quote(self, dataType, data):
        pass


class TestAbsCls(unittest.TestCase):
    def test_instantiate(self):
        class TCls(AbstractStrategy):
            pass

        TCls()



# encoding: utf-8
from __future__ import with_statement
from __future__ import absolute_import
import threading
import time

from communication.server.protocol import function_code
from communication.server.protocol.protocol import ProtocolMessage
import utils


__author__ = u'Yonka'


class RequestInfo(object):
    DEFAULT_TIME_OUT = 1000

    def __init__(
            self,
            protocol_message=None,
    ):
        self.timeout = self.DEFAULT_TIME_OUT
        self.createTime = int(time.time() * 1000)
        if protocol_message is None:
            protocol_message = ProtocolMessage()
        self.protocolMessage = protocol_message
        self._cond = threading.Condition()

    # 注意： 这种场景下，并不在 __dict__ 中，序列化时要注意... = =
    @property
    def reqId(self):
        return self.protocolMessage.reqId

    @reqId.setter
    def reqId(self, req_id):
        # if self.protocolMessage is not None:
        self.protocolMessage.reqId = req_id

    @property
    def fc(self):
        return self.protocolMessage.fc

    @fc.setter
    def fc(self, fc):
        self.protocolMessage.fc = fc

    @property
    def dataType(self):
        return self.protocolMessage.dataType

    @dataType.setter
    def dataType(self, data_type):
        self.protocolMessage.dataType = data_type

    @property
    def data(self):
        return self.protocolMessage.body

    @data.setter
    def data(self, data):
        self.protocolMessage.body = data

    def is_timeout(self):
        return self.timeout > 0 and (self.timeout + self.createTime) < utils.timestamp()

    def wait(self):
        with self._cond:
            self._cond.wait()

    def notify_all(self):
        with self._cond:
            self._cond.notify_all()

    def __str__(self):
        return u"RequestInfo(protocol_message: %s)" % self.protocolMessage


class ResponseInfo(RequestInfo):
    def __init__(
            self,
            # success: bool = False,
            # timed_out: bool = False,
            protocol_message=None
    ):
        super(ResponseInfo, self).__init__(protocol_message)
        self.success = False
        self.timedOut = False

    def is_success(self):
        return self.protocolMessage.fc != function_code.RESP_ST_ORDER



# encoding: utf-8
from __future__ import absolute_import
import logging
import socket
import threading
import time
from ctypes import c_int64

from communication.server import protocol
from communication.client.connection import BaseConn


__author__ = u'Yonka'


class HeartBeatServer(object):
    def __init__(self, addr, port):
        self.addr = addr
        self.port = port
        self.ss = None

    def start(self):
        ss = socket.socket()
        ss.bind((self.addr, self.port))
        ss.listen(5)
        self.ss = ss
        while True:
            s, c_addr = ss.accept()
            logging.info(u"accept conn from: %s", c_addr)
            self.handle_socket(s, c_addr, True)

    def handle_socket(self, s, addr, async=False):
        if async:
            threading.Thread(target=self.handle_socket, args=(s, addr, False)).start()
            return
        base_conn = BaseConn(s)
        last_hb_time = c_int64(int(time.time()))
        self.monitor_conn(base_conn, addr, last_hb_time)
        while True:
            msg = base_conn.read()
            if msg is None:
                logging.info(u"get None msg from base_conn %s, close it", addr)
                base_conn.close()
                break
            if msg.is_hb_message():
                logging.info(u"get hb msg from base_conn %s", addr)
                last_hb_time.value = int(time.time())
            elif msg.is_ack_message():
                logging.info(u"get ack msg from base_conn %s", addr)
                last_hb_time.value = int(time.time())
            else:
                logging.info(u"get non-hb-or-ack msg from base_conn %s: %s", addr, msg)

    def monitor_conn(
            self, conn, addr, last_hb_time,
            monitor_interval=10, hb_send_interval=5, hb_check_interval=1, async=True):
        if async:
            threading.Thread(
                target=self.monitor_conn,
                args=(conn, addr, last_hb_time, monitor_interval, hb_send_interval, hb_check_interval, False)).start()
            return
        while True:
            if time.time() - last_hb_time.value > monitor_interval:
                logging.info(u"send hb to base_conn %s", addr)
                if not conn.write(protocol.new_strategy_hb_msg()):
                    logging.info(u"write hb to base_conn %s failed, close it", addr)
                    conn.close()
                    break
                last_hb_time.value = int(time.time())
                time.sleep(hb_send_interval)
                continue
            time.sleep(hb_check_interval)


if __name__ == u"__main__":
    logging.basicConfig(format=u'%(asctime)s %(message)s', datefmt=u'%m/%d/%Y %I:%M:%S %p', level=logging.INFO)
    HeartBeatServer(u"localhost", 8080).start()



# encoding: utf-8
from __future__ import with_statement
from __future__ import absolute_import
import logging
import threading
import time
import unittest

from concurrent.futures import ThreadPoolExecutor

from communication.client import connection
from communication.server import protocol
from exception.exception import ConnectionBrokenException, RequestTimeoutException
from strategy.client.interface import AbstractStrategyClient
from strategy.request import ResponseInfo
import utils


__author__ = u'Yonka'


class StrategyClient(AbstractStrategyClient):
    MAX_TRIES = 3
    RECONNECT_INTERNAL = 5

    def __init__(
            self,
            host=None,
            port=0,
            read_timeout=0,
            conn_timeout=0,
            async_message_handler=None
    ):
        self.host = host
        self.port = port
        self.readTimeout = read_timeout
        self.connTimeout = conn_timeout
        self.asyncMessageHandler = async_message_handler
        self.open = False
        self.closing = False

        self._lock = threading.Lock()

        self.baseConn = None
        self.ioReaderWorkers = None
        self.bussWorkers = None
        self.heartbeatMonitor = None
        self.timeoutMonitor = None

        self.requestCache = {}
        self.responseCache = {}

    def _build_base_conn(self):
        for i in xrange(self.MAX_TRIES):
            try:
                self.baseConn = connection.new_conn(self.host, self.port, self.readTimeout, self.connTimeout)
                return True
            except Exception, e:
                logging.exception(u"unable to build connection to server %s with port %d e is %s",self.host,self.port, e)
                try:
                    time.sleep(self.RECONNECT_INTERNAL)
                except Exception, e1:
                    logging.exception(u"unable to build connection to server, due to thread interrupt, e is %s", e1)
        return False

    def _write(self, protocol_message):
        if self.open:
            logging.debug("Send message:%s", protocol_message)
            result = self.baseConn.write(protocol_message)
            if not result:
                self.open = False
                raise ConnectionBrokenException(u"connection broken")

    def _do_heartbeat(self):
        try:
            # self._write(protocol.new_strategy_hb_msg())
            self._write(protocol.new_strategy_hb_msg())
        except Exception, e:
            logging.exception(u"%s", e)

    def _check_timeout(self):
        request_cache = dict(self.requestCache)  # 作为client可以
        for req_id, req in request_cache.items():
            if req is None or req_id in self.responseCache:
                continue  # req should not be None
            if req.is_timeout():
                resp = ResponseInfo()
                resp.timedOut = True
                self.responseCache[req_id] = resp
                req.notify_all()

    def _read_conn(self):
        while True:
            try:
                if not self.open:
                    # 这里如果conn被关闭也不重连的话，会一直while循环吃大量CPU
                    time.sleep(1)
                    continue
                protocol_msg = self.baseConn.read()  # type: ProtocolMessage
                if protocol_msg is None:
                    self.open = False
                    continue
                logging.debug("Received message:%s", protocol_msg)

                if protocol_msg.is_hb_message():
                    self._write(protocol.new_strategy_hb_ack_msg())
                    # self._write(protocol.new_strategy_hb_msg())
                else:
                    req = self.requestCache.pop(protocol_msg.reqId, None)  # type: RequestInfo
                    if req is None:
                        self.bussWorkers.submit(
                            lambda async_message: self.asyncMessageHandler.handle_async_message(async_message),
                            protocol_msg
                        )
                    else:
                        self.responseCache[protocol_msg.reqId] = ResponseInfo(protocol_msg)
                        req.notify_all()
            except Exception, e:
                logging.exception(u"_read_conn met exception: %s", e)

    def sync_rpc(self, request_info):
        u"""
        may raise RequestTimeoutException or ConnectionBrokenException
        """
        if not self.is_open():
            raise ConnectionBrokenException(u"socket broken")
        self.requestCache[request_info.reqId] = request_info
        self._write(request_info.protocolMessage)
        request_info.wait()
        resp = self.responseCache.pop(request_info.reqId, None)  # type: ResponseInfo
        if resp is None:
            raise ConnectionBrokenException(u"socket broken")  # should not
        if request_info.is_timeout() or resp.timedOut:
            raise RequestTimeoutException(u"request %d timed out, reqTime: %d, curTime: %d, timeout: %d, req is %s" % (
                request_info.reqId, request_info.createTime, utils.timestamp(), request_info.timeout, request_info))
        return resp

    def close(self):
        with self._lock:
            self.closing = True
            self.open = False

            for req in self.requestCache.values():
                req.notify_all()

            if self.baseConn is not None and self.baseConn.is_open():
                self.baseConn.close()
                self.baseConn = None

            if self.ioReaderWorkers is not None:
                self.ioReaderWorkers.shutdown()
            if self.timeoutMonitor is not None:
                self.timeoutMonitor.shutdown()
            if self.heartbeatMonitor is not None:
                self.heartbeatMonitor.shutdown()

            self.requestCache = {}
            self.responseCache = {}

        self.closing = False
        return True

    def connect(self):
        self.ioReaderWorkers = ThreadPoolExecutor(1)
        self.bussWorkers = ThreadPoolExecutor(1)
        if not self._build_base_conn():
            logging.warn(u"Socket connection timeout")
            self.open = False
            raise Exception(u"Socket connection timeout")  # XXX use ConnectionBrokenException ?
        else:
            logging.debug(u"Socket connection open")
            self.open = True

        self.heartbeatMonitor = ThreadPoolExecutor(1)
        self.heartbeatMonitor.submit(utils.schedule_task(self._do_heartbeat, 5, 10, 0))

        self.timeoutMonitor = ThreadPoolExecutor(1)
        self.timeoutMonitor.submit(utils.schedule_task(self._check_timeout, 1, 1, 0))

        self.ioReaderWorkers.submit(self._read_conn)

        return True

    def is_open(self):
        return self.open and not self.closing

    def async_rpc(self, request_info):
        if not self.is_open():
            raise ConnectionBrokenException(u"socket broken")
        self._write(request_info.protocolMessage)
        return True


class TestClient(unittest.TestCase):
    def test_client(self):
        logging.basicConfig(format=u'%(asctime)s %(message)s', datefmt=u'%m/%d/%Y %I:%M:%S %p', level=logging.INFO)
        c = StrategyClient(u"101.200.228.73", 8080)
        # c = StrategyClient("101.200.228.73", 3333)
        # c = StrategyClient("localhost", 8080)
        c.connect()
        time.sleep(10000)



# encoding: utf-8
from __future__ import absolute_import
import unittest

from utils.enum import EnumMetaCls, CommonNameValueEnum, can_not_init

__author__ = u'yonka'


class InterfaceType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"InterfaceType(%d)" % self.v


InterfaceType.INTERFACE_FUTURE = InterfaceType("1")
InterfaceType.INTERFACE_OPTION = InterfaceType("2")
InterfaceType.INTERFACE_STOCK = InterfaceType("3")

InterfaceType.__init__ = can_not_init


class OperationType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"OperationType(%d)" % self.v


OperationType.OPT_INVALID = OperationType(-1)
OperationType.OPT_OPEN_LONG = OperationType(0)#---support
OperationType.OPT_CLOSE_LONG_HISTORY = OperationType(1)#---support
OperationType.OPT_CLOSE_LONG_TODAY = OperationType(2)#---support
OperationType.OPT_OPEN_SHORT = OperationType(3)#---support
OperationType.OPT_CLOSE_SHORT_HISTORY = OperationType(4)#---support
OperationType.OPT_CLOSE_SHORT_TODAY = OperationType(5)#---support
OperationType.OPT_CLOSE_LONG_TODAY_FIRST = OperationType(6)
OperationType.OPT_CLOSE_LONG_HISTORY_FIRST = OperationType(7)
OperationType.OPT_CLOSE_SHORT_TODAY_FIRST = OperationType(8)
OperationType.OPT_CLOSE_SHORT_HISTORY_FIRST = OperationType(9)
OperationType.OPT_CLOSE_LONG_TODAY_HISTORY_THEN_OPEN_SHORT = OperationType(10)
OperationType.OPT_CLOSE_LONG_HISTORY_TODAY_THEN_OPEN_SHORT = OperationType(11)
OperationType.OPT_CLOSE_SHORT_TODAY_HISTORY_THEN_OPEN_LONG = OperationType(12)
OperationType.OPT_CLOSE_SHORT_HISTORY_TODAY_THEN_OPEN_LONG = OperationType(13)
OperationType.OPT_CLOSE_LONG = OperationType(14)#---support
OperationType.OPT_CLOSE_SHORT = OperationType(15)#---support
OperationType.OPT_OPEN = OperationType(16)
OperationType.OPT_CLOSE = OperationType(17)
# ############# 以上是期货的操作
# ############# 以下是股票的操作
OperationType.OPT_BUY = OperationType(18)#---support
OperationType.OPT_SELL = OperationType(19)#---support
OperationType.OPT_FIN_BUY = OperationType(20)
OperationType.OPT_SLO_SELL = OperationType(21)
OperationType.OPT_BUY_SECU_REPAY = OperationType(22)
OperationType.OPT_DIRECT_SECU_REPAY = OperationType(23)
OperationType.OPT_SELL_CASH_REPAY = OperationType(24)
OperationType.OPT_DIRECT_CASH_REPAY = OperationType(25)
OperationType._C_OPT_COUNT = OperationType(26)

OperationType.__init__ = can_not_init


class PriceType(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"PriceType(%d)" % self.v


PriceType.PRTP_INVALID = PriceType(-1)
PriceType.PRTP_SALE5 = PriceType(0)
PriceType.PRTP_SALE4 = PriceType(1)
PriceType.PRTP_SALE3 = PriceType(2)
PriceType.PRTP_SALE2 = PriceType(3)
PriceType.PRTP_SALE1 = PriceType(4)
PriceType.PRTP_LATEST = PriceType(5)
PriceType.PRTP_BUY1 = PriceType(6)
PriceType.PRTP_BUY2 = PriceType(7)
PriceType.PRTP_BUY3 = PriceType(8)
PriceType.PRTP_BUY4 = PriceType(9)
PriceType.PRTP_BUY5 = PriceType(10)
PriceType.PRTP_FIX = PriceType(11)
PriceType.PRTP_MARKET = PriceType(12)
PriceType.PRTP_HANG = PriceType(13)
PriceType.PRTP_COMPETE = PriceType(14)
PriceType._C_PRTP_COUNT = PriceType(15)

PriceType.__init__ = can_not_init


# class QuoteDataType(CommonNameValueEnum):
# __metaclass__ = EnumMetaCls
#
# def __str__(self):
#         return u"QuoteDataType(%d)" % self.v

# #LEVEL 1 tick级别行情
# QuoteDataType.TICK_LEVEL_ONE_STOCK=0    #股票tick行情
# QuoteDataType.TICK_LEVEL_ONE_FUTURE=QuoteDataType(1)    #期货tick行情
# QuoteDataType.TICK_LEVEL_ONE_OPTION=QuoteDataType(2)    #期权tick行情
# #LEVEL 2 tick级别行情
# QuoteDataType.TICK_LEVEL_TWO_STOCK=QuoteDataType(10) #    //股票tick行情
# QuoteDataType.TICK_LEVEL_TWO_FUTURE=QuoteDataType(12)#,   //期货tick行情
# QuoteDataType.TICK_LEVEL_TWO_OPTION=QuoteDataType(13)#,   //期权tick
#
# QuoteDataType.KLINE=QuoteDataType(20)
#
# QuoteDataType.TRANSACTION=QuoteDataType(30)
# QuoteDataType.ORDER=QuoteDataType(40)#,//订单成交
# QuoteDataType.ORDER_QUEUE=QuoteDataType(50)#,//订单队列
#
#
# QuoteDataType.SAMPLE_DATA_BEGIN=QuoteDataType(100)#,
# QuoteDataType.SAMPLE_DATA_SECOND=QuoteDataType(101)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_MINUTE=QuoteDataType(102)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_DAY=QuoteDataType(103)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_WEEK=QuoteDataType(104)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_MONTH=QuoteDataType(105)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_SEASON=QuoteDataType(106)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_YEAR=QuoteDataType(107)#,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
# QuoteDataType.SAMPLE_DATA_END=QuoteDataType(200)

# QuoteDataType.__init__ = can_not_init


class QuoteMarket(CommonNameValueEnum):
    __metaclass__ = EnumMetaCls

    def __str__(self):
        return u"QuoteMarket(%d)" % self.v


QuoteMarket.SHFE = QuoteMarket(0)
QuoteMarket.DCE = QuoteMarket(4)
QuoteMarket.CZCE = QuoteMarket(6)
QuoteMarket.CFFEX = QuoteMarket(8)
QuoteMarket.SHSE = QuoteMarket(101)
QuoteMarket.SZSE = QuoteMarket(102)

QuoteMarket.__init__ = can_not_init


class TestEnum(unittest.TestCase):
    def test_init_interface_type(self):
        print InterfaceType.INTERFACE_FUTURE
        res = False
        try:
            InterfaceType(1)
        except AssertionError:
            res = True
        assert res

    def test_init_operation_type(self):
        print OperationType.OPT_INVALID
        res = False
        try:
            OperationType(1)
        except AssertionError:
            res = True
        assert res

    def test_init_price_type(self):
        print PriceType.PRTP_INVALID
        res = False
        try:
            PriceType(1)
        except AssertionError:
            res = True
        assert res

    def test_init_quote_data_type(self):
        print QuoteDataType.TICK_FIVE_LEVEL
        res = False
        try:
            QuoteDataType(0)
        except AssertionError:
            res = True
        assert res

    def test_init_quote_market(self):
        print QuoteMarket.SHFE
        res = False
        try:
            QuoteMarket(0)
        except AssertionError:
            res = True
        assert res



# encoding: utf-8

TICK_LEVEL_ONE_STOCK = 0  # 股票tick行情
TICK_LEVEL_ONE_FUTURE = 1  # 期货tick行情
TICK_LEVEL_ONE_OPTION = 2  # 期权tick行情
TICK_FUTURE = 6  # 期货tick行情
TICK_OPTION = 7  # 期权tick行情
TICK_STOCK = 8  # 股票tick行情
# LEVEL 2 tick级别行情
TICK_LEVEL_TWO_STOCK = 10  # //股票tick行情
TICK_LEVEL_TWO_FUTURE = 12  # ,   //期货tick行情
TICK_LEVEL_TWO_OPTION = 13  # ,   //期权tick

KLINE = 20
KLINE_ONE_MINUTE = 21       #1分钟线
KLINE_FIVE_MINUTE = 22      #5分钟线
KLINE_TEN_MINUTE = 23       #10分钟
KLINE_FIFTEEN_MINUTE = 24   #15分钟
KLINE_THIRTY_MINUTE = 25    #30分钟
KLINE_ONE_HOUR = 26         #1小时


KLINE_ONE_SECOND = 36
KLINE_FIVE_SECOND = 37
KLINE_TEN_SECOND = 38
KLINE_THIRTY_SECOND = 39

KLINE_END = 50

TRANSACTION = 60
ORDER = 70  # ,//订单成交
ORDER_QUEUE = 80  # ,//订单队列
###############以上是订阅的行情
##############以下是查询的行情
SAMPLE_DATA_BEGIN = 100  # ,
SAMPLE_DATA_SECOND = 101  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_MINUTE = 102  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_DAY = 103  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_WEEK = 104  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_MONTH = 105  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_SEASON = 106  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_YEAR = 107  # ,//采样数据（采样的类型通过请求参数确定,通过采样种类，采样间隔，是否填充来决定）
SAMPLE_DATA_END = 200


# encoding: utf-8
from __future__ import absolute_import
import codecs
import os

try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup
u"""
打包的用的setup必须引入，
"""

__author__ = u'Yonka'


def read(fname):
    u"""
    定义一个read方法，用来读取目录下的长描述
    我们一般是将README文件中的内容读取出来作为长描述，这个会在PyPI中你这个包的页面上展现出来，
    你也可以不用这个方法，自己手动写内容即可，
    PyPI上支持.rst格式的文件。暂不支持.md格式的文件，<BR>.rst文件PyPI会自动把它转为HTML形式显示在你包的信息页面上。
    """
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


NAME = u"strategy_sdk_py2"
u"""
名字，一般放你包的名字即可
"""

PACKAGES = find_packages("strategy_sdk")
print PACKAGES
u"""
包含的包，可以多个，这是一个列表
"""

DESCRIPTION = u"this is a python2 client sdk for strategy access"
u"""
关于这个包的描述
"""

LONG_DESCRIPTION = read(u"README.rst")
u"""
参见read方法说明
"""

KEYWORDS = u"python2 strategy sdk client"
u"""
关于当前包的一些关键字，方便PyPI进行分类。
"""

AUTHOR = u"vector tang"
u"""
谁是这个包的作者，写谁的名字吧
我是MitchellChu，自然这里写的是MitchellChu
"""

AUTHOR_EMAIL = u"tangtao@hotmail.com"
u"""
作者的邮件地址
"""

URL = u"https://pypi.python.org/pypi/strategy-sdk-py2"
u"""
你这个包的项目地址，如果有，给一个吧，没有你直接填写在PyPI你这个包的地址也是可以的
"""

VERSION = u"1.0.6"
u"""
当前包的版本，这个按你自己需要的版本控制方式来
"""

LICENSE = u"MIT"
u"""
授权方式，我喜欢的是MIT的方式，你可以换成其他方式
"""
install_requires=[
    'protobuf',
]
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        u'License :: OSI Approved :: MIT License',
        u'Programming Language :: Python',
        u'Intended Audience :: Developers',
        u'Operating System :: OS Independent',
    ],
    keywords=KEYWORDS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license=LICENSE,
    packages=PACKAGES,
    package_dir={u"": "strategy_sdk"},
    include_package_data=True,
    zip_safe=True,
    requires=[
        'protobuf'
    ]
)

# 把上面的变量填入了一个setup()中即可。



# encoding: utf-8
from __future__ import absolute_import
import unittest

from data.common.enum import OffsetFlagType, EntrustSubmitStatus, \
    EntrustStatus, HedgeFlagType, EntrustBS
from agent import agent_type
from strategy.bean.enum import OperationType, PriceType, InterfaceType
from utils.bean import BaseBean
from utils.typing import List

__author__ = u'yonka'


class StrategyFinanceAccountDetail(BaseBean):
    def __init__(
            self,
            account_id=None,
            status=None,
            open_date=None,
            trading_date=None,
            available_money=0,
            instrument_value=0,
            balance=0,
            margin_rate=0,
            frozen_margin=0,
            frozen_cash=0,
            frozen_commission=0,
            risk_rate=0,
            net_value=0,
            pre_balance=0,
            commission=0,
            position_profit=0,
            close_profit=0,
            curr_margin=0,
            init_close_money=0,
            deposit=0,
            withdraw=0,
            pre_credit=0,
            pre_mortgage=0,
            credit=0,
            mortgage=0,
            assure_asset=0,
            entrust_asset=0,
            total_debit=0,
    ):
        self.accountId = account_id
        self.status = status
        self.openDate = open_date
        self.tradingDate = trading_date
        self.availableMoney = available_money
        self.instrumentValue = instrument_value
        self.balance = balance
        self.marginRate = margin_rate
        self.frozenMargin = frozen_margin
        self.frozenCash = frozen_cash
        self.frozenCommission = frozen_commission
        self.riskRate = risk_rate
        self.netValue = net_value
        self.preBalance = pre_balance
        self.commission = commission
        self.positionProfit = position_profit
        self.closeProfit = close_profit
        self.currMargin = curr_margin
        self.initCloseMoney = init_close_money
        self.deposit = deposit
        self.withdraw = withdraw
        self.preCredit = pre_credit
        self.preMortgage = pre_mortgage
        self.credit = credit
        self.mortgage = mortgage
        self.assureAsset = assure_asset
        self.entrustAsset = entrust_asset
        self.totalDebit = total_debit


class StrategyPositionDetail(BaseBean):
    _types = {
        u"hedgeFlag": HedgeFlagType,
        u"direction": EntrustBS,
    }

    def __init__(
            self,
            account_id=None,
            exchange_id=None,
            exchange_name=None,
            product_id=None,
            product_name=None,
            instrument_id=None,
            instrument_name=None,
            open_date=None,
            trade_id=None,
            volume=0,
            open_price=0,
            trading_day=None,
            margin=0,
            open_cost=0,
            settlement_price=0,
            close_volume=0,
            close_amount=0,
            float_profit=0,
            close_profit=0,
            market_value=0,
            position_cost=0,
            position_profit=0,
            last_settlement_price=0,
            instrument_value=0,
            is_today=False,
            order_id=None,
            frozen_volume=0,
            can_use_volume=0,
            on_road_volume=0,
            yesterday_volume=0,
            last_price=0,
            profit_rate=0,
            hedge_flag=None,
            direction=None,
    ):
        self.accountID = account_id,
        self.exchangeID = exchange_id,
        self.exchangeName = exchange_name,
        self.productID = product_id,
        self.productName = product_name,
        self.instrumentID = instrument_id,
        self.instrumentName = instrument_name,
        self.openDate = open_date,
        self.tradeID = trade_id,
        self.volume = volume,
        self.openPrice = open_price,
        self.tradingDay = trading_day,
        self.margin = margin,
        self.openCost = open_cost,
        self.settlementPrice = settlement_price,
        self.closeVolume = close_volume,
        self.closeAmount = close_amount,
        self.floatProfit = float_profit,
        self.closeProfit = close_profit,
        self.marketValue = market_value,
        self.positionCost = position_cost,
        self.positionProfit = position_profit,
        self.lastSettlementPrice = last_settlement_price,
        self.instrumentValue = instrument_value,
        self.isToday = is_today,
        self.orderID = order_id,
        self.frozenVolume = frozen_volume,
        self.canUseVolume = can_use_volume,
        self.onRoadVolume = on_road_volume,
        self.yesterdayVolume = yesterday_volume,
        self.lastPrice = last_price,
        self.profitRate = profit_rate,
        self.hedgeFlag = hedge_flag,
        self.direction = direction,


class StrategyPositionStatics(BaseBean):
    def __init__(
            self,
            account_id=None,
            exchange_id=None,
            exchange_name=None,
            product_id=None,
            instrument_id=None,
            instrument_name=None,
            yesterday_position=0,
            today_position=0,
            open_cost=0,
            position_cost=0,
            close_profit=0,
            float_profit=0,
            open_price=0,
            can_close_vol=0,
            used_margin=0,
            used_commission=0,
            frozen_margin=0,
            frozen_commission=0,
            instrument_value=0,
            open_times=0,
            open_volume=0,
            cancel_times=0,
            frozen_volume=0,
            can_use_volume=0,
            on_road_volume=0,
            settlement_price=0,
            profit_rate=0,
            hedge_flag=None,
            direction=None,
            create_time=None,
    ):
        self.accountID = account_id
        self.exchangeID = exchange_id
        self.exchangeName = exchange_name
        self.productID = product_id
        self.instrumentID = instrument_id
        self.instrumentName = instrument_name
        self.yesterdayPosition = yesterday_position
        self.todayPosition = today_position
        self.openCost = open_cost
        self.positionCost = position_cost
        self.closeProfit = close_profit
        self.floatProfit = float_profit
        self.openPrice = open_price
        self.canCloseVol = can_close_vol
        self.usedMargin = used_margin
        self.usedCommission = used_commission
        self.frozenMargin = frozen_margin
        self.frozenCommission = frozen_commission
        self.instrumentValue = instrument_value
        self.openTimes = open_times
        self.openVolume = open_volume
        self.cancelTimes = cancel_times
        self.frozenVolume = frozen_volume
        self.canUseVolume = can_use_volume
        self.onRoadVolume = on_road_volume
        self.settlementPrice = settlement_price
        self.profitRate = profit_rate
        self.hedgeFlag = hedge_flag
        self.direction = direction
        self.createTime = create_time


class CtpOrderError(BaseBean):
    def __init__(
            self,
            account_id=None,
            error_code=0,
            error_msg=None,
            session_id=None,
            front_id=None,
            order_ref_id=None,
            request_id=0,
            order_id=None,
            order_sys_id=None,
    ):
        self.accountId = account_id
        self.errorCode = error_code
        self.errorMsg = error_msg
        self.sessionId = session_id
        self.front_id = front_id
        self.orderRefId = order_ref_id
        self.requestId = request_id
        self.orderId = order_id
        self.orderSysId = order_sys_id


class StrategyAccountDetail(BaseBean):
    def __init__(
            self,
            account_id=None,
            status=None,
            open_date=None,
            trading_date=None,
            available_money=0,
            instrument_value=0,
            balance=0,
            margin_rate=0,
            frozen_margin=0,
            frozen_cash=0,
            frozen_commission=0,
            risk_rate=0,
            net_value=0,
            pre_balance=0,
            commission=0,
            position_profit=0,
            close_profit=0,
            curr_margin=0,
            init_balance=0,
            init_close_money=0,
            deposit=0,
            withdraw=0,
            pre_credit=0,
            pre_mortgage=0,
            credit=0,
            mortgage=0,
            assure_asset=0,
            entrust_asset=0,
            total_debit=0
    ):
        self.accountId = account_id
        self.status = status
        self.openDate = open_date
        self.tradingDate = trading_date
        self.availableMoney = available_money
        self.instrumentValue = instrument_value
        self.balance = balance
        self.marginRate = margin_rate
        self.frozenMargin = frozen_margin
        self.frozenCash = frozen_cash
        self.frozenCommission = frozen_commission
        self.riskRate = risk_rate
        self.netValue = net_value
        self.preBalance = pre_balance
        self.commission = commission
        self.positionProfit = position_profit
        self.closeProfit = close_profit
        self.currMargin = curr_margin
        self.initBalance = init_balance
        self.initCloseMoney = init_close_money
        self.deposit = deposit
        self.withdraw = withdraw
        self.preCredit = pre_credit
        self.preMortgage = pre_mortgage
        self.credit = credit
        self.mortgage = mortgage
        self.assureAsset = assure_asset
        self.entrustAsset = entrust_asset
        self.totalDebit = total_debit


class StrategyError(BaseBean, RuntimeError):
    def __init__(
            self,
            error_id=0,
            error_msg=None
    ):
        self.errorId = error_id
        self.errorMsg = error_msg


# class StrategyAccountDetailResultBean(BaseBean):
# _types = {
# u"data": ArrayStrategyFinanceAccountDetails,
# u"error": StrategyError
#     }
#
#     def __init__(
#             self,
#             data=None,
#             error=None
#     ):
#         self.data = data
#         self.error = error


class StrategyConnectionInfo(BaseBean):
    def __init__(
            self,
            ip=None,
            port=None,
            account_id=None,
            strategy_id=None,
            password=None,
            strategy_client_id=None
    ):
        self.ip = ip
        self.port = port
        self.accountId = account_id
        self.strategyId = strategy_id
        self.password = password
        self.strategyClientID = strategy_client_id


class QuoteDataCategory(BaseBean):
    def __init__(
            self,
            data_symbol=None,
            begin_time=None,
            end_time=None,
    ):
        self.dataSymbol = data_symbol
        self.beginTime = begin_time
        self.endTime = end_time
        # //////////确定数据类型////////////////
        # public String market;
        # public String symbol;
        # //数据周期：tick,秒线、分钟、日线、周线、月线、季线、半年线、年线,见QuoteDataType
        # public int dataType;//数据类型
        # //周期数量：dataType取值为：秒、分钟、日线、周线、月线时，这个字段有效。
        # public int cycCount;
        # //自动填充：仅1秒钟线、1分钟线支持这个标志，（1：补齐；0：不补齐）
        # public int autoFill;
        # //开始日期(交易日，<0:从上市日期开始； 0:从今天开始)
        # public int beginDate;
        # //结束日期(交易日，<=0:跟nBeginDate一样)
        # public int endDate;
        # //开始时间，<=0表示从开始，格式：（HHMMSSmmm）例如94500000 表示 9点45分00秒000毫秒
        # public int beginTime;
        # //结束时间，<=0表示到结束，格式：（HHMMSSmmm）例如94500000 表示 9点45分00秒000毫秒
        # public int endTime;
        #
        # ////////////除权复权标识，只有在数据请求的时候有效///////////////
        #
        # //除权标志：不复权，向前复权，向后复权
        # public int cqFlag;
        # //复权日期(<=0:全程复权) 格式：YYMMDD，例如20130101表示2013年1月1日
        # public int cqDate;
        # //全价标志(债券)(0:净价 1:全价)
        # public int qjFlag;


class StrategyQuoteDataRequest(BaseBean):
    def __init__(
            self,
            market="",
            symbol=None,
            dataType=None,
            cycCount=None,
            autoFill=0,
            beginDate=-1,
            endDate=-1,
            beginTime=-1,
            endTime=-1,
            cqFlag=0,
            cqDate=-1,
            qjFlag=0
    ):
        self.market = market
        self.symbol = symbol
        self.dataType = dataType
        self.cycCount = cycCount
        self.autoFill = autoFill
        self.beginDate = beginDate
        self.endDate = endDate
        self.beginTime = beginTime
        self.endTime = endTime
        self.cqFlag = cqFlag
        self.cqDate = cqDate
        self.qjFlag = qjFlag


class StrategyCancelOrder(BaseBean):
    def __init__(
            self,
            account_id=None,
            session_id=None,
            front_id=None,
            order_ref_id=None,
            request_id=0,
            order_id=None,
    ):
        self.accountId = account_id
        self.sessionId = session_id
        self.frontId = front_id
        self.orderRefId = order_ref_id
        self.requestId = request_id
        self.orderId = order_id


class StrategyConfiguration(BaseBean):
    _types = {
        u"strategyConnectionInfo": StrategyConnectionInfo
    }

    def __init__(
            self,
            host=None,
            port=0,
            read_timeout=0,
            conn_timeout=0,
            usage_type=0,
            strategy_connection_info=None,
    ):
        self.host = host
        self.port = port
        self.readTimeout = read_timeout
        self.connTimeout = conn_timeout
        self.usageType = usage_type
        self.strategyConnectionInfo = strategy_connection_info


class StrategyOrder(BaseBean):
    _types = {
        u"m_ePriceType": PriceType,
        u"m_eOperationType": OperationType,
        u"m_eHedgeFlag": HedgeFlagType
    }

    def __init__(
            self,
            m_str_account_id=None,
            m_s_strategy_id=None,
            m_d_price=0,
            m_d_super_price=0,
            m_n_volume=0,
            m_str_market=None,
            m_str_product=None,
            m_str_instrument=None,
            m_e_price_type=None,
            m_e_operation_type=None,
            m_e_hedge_flag=None
    ):
        self.m_strAccountID = m_str_account_id
        self.m_sStrategyID = m_s_strategy_id
        self.m_dPrice = m_d_price
        self.m_dSuperPrice = m_d_super_price
        self.m_nVolume = m_n_volume
        self.m_strMarket = m_str_market
        self.m_strProduct = m_str_product
        self.m_strInstrument = m_str_instrument
        self.m_ePriceType = m_e_price_type
        self.m_eOperationType = m_e_operation_type
        self.m_eHedgeFlag = m_e_hedge_flag


class StrategyOrderCancel(BaseBean):
    def __init__(
            self,
            account_id=None,
            request_id=0,
            order_id=None
    ):
        self.accountId = account_id
        self.requestId = request_id
        self.orderId = order_id


class StrategyOrderCancelError(BaseBean):
    def __init__(
            self,
            account_id=None,
            error_code=0,
            error_msg=None,
            request_id=0,
            order_id=None
    ):
        self.accountId = account_id
        self.errorCode = error_code
        self.errorMsg = error_msg
        self.requestId = request_id
        self.orderId = order_id


class StrategyOrderDetail(BaseBean):
    def __init__(
            self,
            account_id=None,
            exchange_id=None,
            exchange_name=None,
            product_id=None,
            product_name=None,
            instrument_id=None,
            instrument_name=None,
            limit_price=0,
            volume_total_original=0,
            order_id=None,
            volume_traded=0,
            volume_total=0,
            frozen_margin=0,
            frozen_commission=0,
            traded_price=0,
            cancel_amount=0,
            trade_amount=0,
            error_code=0,
            error_msg=None,
            insert_date=None,
            insert_time=None,
            opt_name=None,
            direction=None,
            offset_flag=None,
            hedge_flag=None,
            order_price_type=None,
            order_submit_status=None,
            order_status=None
    ):
        self.accountId = account_id
        self.exchangeId = exchange_id
        self.exchangeName = exchange_name
        self.productId = product_id
        self.productName = product_name
        self.instrumentId = instrument_id
        self.instrumentName = instrument_name
        self.limitPrice = limit_price
        self.volumeTotalOriginal = volume_total_original
        self.orderId = order_id
        self.volumeTraded = volume_traded
        self.volumeTotal = volume_total
        self.frozenMargin = frozen_margin
        self.frozenCommission = frozen_commission
        self.tradedPrice = traded_price
        self.cancelAmount = cancel_amount
        self.tradeAmount = trade_amount
        self.errorCode = error_code
        self.errorMsg = error_msg
        self.insertDate = insert_date
        self.insertTime = insert_time
        self.optName = opt_name
        self.direction = direction
        self.offsetFlag = offset_flag
        self.hedgeFlag = hedge_flag
        self.orderPriceType = order_price_type
        self.orderSubmitStatus = order_submit_status
        self.orderStatus = order_status


class StrategyOrderError(BaseBean):
    def __init__(
            self,
            account_id=None,
            error_code=0,
            error_msg=None,
            request_id=0,
            order_id=None
    ):
        self.accountId = account_id
        self.errorCode = error_code
        self.errorMsg = error_msg
        self.requestId = request_id
        self.orderId = order_id


# class StrategyPositionDetailResultBean(BaseBean):
#     _types = {
#         u"data": ArrayStrategyPositionDetails,
#         u"error": StrategyError
#     }
#
#     def __init__(
#             self,
#             data=None,
#             error=None
#     ):
#         self.data = data
#         self.error = error


# class StrategyPositionStaticsResultBean(BaseBean):
#     _types = {
#         u"data": ArrayStrategyPositionStatics,
#         u"strategy_error": StrategyError
#     }
#
#     def __init__(
#             self,
#             data=None,
#             strategy_error=None
#     ):
#         self.data = data
#         self.strategyError = strategy_error


class StrategyPriceData(BaseBean):
    def __init__(
            self,
            trading_day=None,
            exchange_id=None,
            instrument_id=None,
            instrument_name=None,
            exchange_inst_id=None,
            last_price=0,
            up_down=0,
            up_down_rate=0,
            average_price=0,
            volume=0,
            turnover=0,
            pre_close_price=0,
            pre_settlement_price=0,
            pre_open_interest=0,
            open_interest=0,
            settlement_price=0,
            open_price=0,
            highest_price=0,
            lowest_price=0,
            close_price=0,
            upper_limit_price=0,
            lower_limit_price=0,
            pre_delta=0,
            curr_delta=0,
            update_time=None,
            update_millisec=0,
            bid_price1=0,
            bid_volume1=0,
            ask_price1=0,
            ask_volume1=0,
            bid_price2=0,
            bid_volume2=0,
            ask_price2=0,
            ask_volume2=0,
            bid_price3=0,
            bid_volume3=0,
            ask_price3=0,
            ask_volume3=0,
            bid_price4=0,
            bid_volume4=0,
            ask_price4=0,
            ask_volume4=0,
            bid_price5=0,
            bid_volume5=0,
            ask_price5=0,
            ask_volume5=0,
            bid_price6=0,
            bid_volume6=0,
            ask_price6=0,
            ask_volume6=0,
            pre_price=0
    ):
        self.tradingDay = trading_day
        self.exchangeId = exchange_id
        self.instrumentId = instrument_id
        self.instrumentName = instrument_name
        self.exchangeInstID = exchange_inst_id
        self.lastPrice = last_price
        self.upDown = up_down
        self.upDownRate = up_down_rate
        self.averagePrice = average_price
        self.volume = volume
        self.turnover = turnover
        self.preClosePrice = pre_close_price
        self.preSettlementPrice = pre_settlement_price
        self.preOpenInterest = pre_open_interest
        self.openInterest = open_interest
        self.settlementPrice = settlement_price
        self.openPrice = open_price
        self.highestPrice = highest_price
        self.lowestPrice = lowest_price
        self.closePrice = close_price
        self.upperLimitPrice = upper_limit_price
        self.lowerLimitPrice = lower_limit_price
        self.preDelta = pre_delta
        self.currDelta = curr_delta
        self.updateTime = update_time
        self.updateMillisec = update_millisec
        self.bidPrice1 = bid_price1
        self.bidVolume1 = bid_volume1
        self.askPrice1 = ask_price1
        self.askVolume1 = ask_volume1
        self.bidPrice2 = bid_price2
        self.bidVolume2 = bid_volume2
        self.askPrice2 = ask_price2
        self.askVolume2 = ask_volume2
        self.bidPrice3 = bid_price3
        self.bidVolume3 = bid_volume3
        self.askPrice3 = ask_price3
        self.askVolume3 = ask_volume3
        self.bidPrice4 = bid_price4
        self.bidVolume4 = bid_volume4
        self.askPrice4 = ask_price4
        self.askVolume4 = ask_volume4
        self.bidPrice5 = bid_price5
        self.bidVolume5 = bid_volume5
        self.askPrice5 = ask_price5
        self.askVolume5 = ask_volume5
        self.bidPrice6 = bid_price6
        self.bidVolume6 = bid_volume6
        self.askPrice6 = ask_price6
        self.askVolume6 = ask_volume6
        self.prePrice = pre_price


# class StrategyQuoteDataRequest(BaseBean):
#     def __init__(
#             self,
#             data_type=None,
#             begin_date=None,
#             end_date=None
#     ):
#         self.dataType = data_type
#         self.beginDate = begin_date
#         self.endDate = end_date


class StrategyQuoteDataResult(BaseBean):
    _types = {
        u"strategyError": StrategyError
    }

    def __init__(
            self,
            strategy_error=None,
            data_type=None,
            data=None
    ):
        self.strategyError = strategy_error
        self.dataType = data_type
        self.data = data


class StrategyRequestOrder(BaseBean):
    _types = {
        u"priceType": PriceType,
        u"hedgeFlag": HedgeFlagType,
        u"orderOperationType": OperationType,
        u"interfaceType": InterfaceType
    }

    def __init__(
            self,
            account_id=None,
            price=0,
            volume=0,
            strategy_id=None,
            market=None,
            product=None,
            instrument=None,
            price_type=None,
            hedge_flag=None,
            order_operation_type=None,
            interface_type=None
    ):
        self.accountId = account_id
        self.price = price
        self.volume = volume
        self.strategyId = strategy_id
        self.market = market
        self.product = product
        self.instrument = instrument
        self.priceType = price_type
        self.hedgeFlag = hedge_flag
        self.orderOperationType = order_operation_type
        self.interfaceType = interface_type
        self.strategyClientID = None  # type: str


class StrategyTradeDetail(BaseBean):
    _types = {
        u"direction": EntrustBS,
        u"offsetFlag": OffsetFlagType,
        u"hedgeFlag": HedgeFlagType,
        u"orderPriceType": PriceType
    }

    def __init__(
            self,
            account_id=None,
            exchange_id=None,
            exchange_name=None,
            product_id=None,
            product_name=None,
            instrument_id=None,
            instrument_name=None,
            trade_id=None,
            order_id=None,
            price=0,
            volume=0,
            trade_date=None,
            trade_time=None,
            comssion=0,  # FIXME 可能原java代码拼写错误
            trade_amount=0,
            opt_name=None,
            direction=None,
            offset_flag=None,
            hedge_flag=None,
            order_price_type=None
    ):
        self.accountId = account_id
        self.exchangeId = exchange_id
        self.exchangeName = exchange_name
        self.productID = product_id
        self.productName = product_name
        self.instrumentId = instrument_id
        self.instrumentName = instrument_name
        self.tradeId = trade_id
        self.orderId = order_id
        self.price = price
        self.volume = volume
        self.tradeDate = trade_date
        self.tradeTime = trade_time
        self.comssion = comssion
        self.tradeAmount = trade_amount
        self.optName = opt_name
        self.direction = direction
        self.offsetFlag = offset_flag
        self.hedgeFlag = hedge_flag
        self.orderPriceType = order_price_type


class TickFiveLevelData(BaseBean):
    def __init__(
            self,
            trading_day=None,
            exchange_id=None,
            instrument_id=None,
            instrument_name=None,
            exchange_inst_id=None,
            last_price=0,
            up_down=0,
            up_down_rate=0,
            average_price=0,
            volume=0,
            turnover=0,
            pre_close_price=0,
            pre_settlement_price=0,
            pre_open_interest=0,
            open_interest=0,
            settlement_price=0,
            open_price=0,
            highest_price=0,
            lowest_price=0,
            close_price=0,
            upper_limit_price=0,
            lower_limit_price=0,
            pre_delta=0,
            curr_delta=0,
            update_time=None,
            update_millisec=0,
            bid_price1=0,
            bid_volume1=0,
            ask_price1=0,
            ask_volume1=0,
            bid_price2=0,
            bid_volume2=0,
            ask_price2=0,
            ask_volume2=0,
            bid_price3=0,
            bid_volume3=0,
            ask_price3=0,
            ask_volume3=0,
            bid_price4=0,
            bid_volume4=0,
            ask_price4=0,
            ask_volume4=0,
            bid_price5=0,
            bid_volume5=0,
            ask_price5=0,
            ask_volume5=0,
            bid_price6=0,
            bid_volume6=0,
            ask_price6=0,
            ask_volume6=0,
            pre_price=0
    ):
        self.tradingDay = trading_day
        self.exchangeId = exchange_id
        self.instrumentId = instrument_id
        self.instrumentName = instrument_name
        self.exchangeInstID = exchange_inst_id
        self.lastPrice = last_price
        self.upDown = up_down
        self.upDownRate = up_down_rate
        self.averagePrice = average_price
        self.volume = volume
        self.turnover = turnover
        self.preClosePrice = pre_close_price
        self.preSettlementPrice = pre_settlement_price
        self.preOpenInterest = pre_open_interest
        self.openInterest = open_interest
        self.settlementPrice = settlement_price
        self.openPrice = open_price
        self.highestPrice = highest_price
        self.lowestPrice = lowest_price
        self.closePrice = close_price
        self.upperLimitPrice = upper_limit_price
        self.lowerLimitPrice = lower_limit_price
        self.preDelta = pre_delta
        self.currDelta = curr_delta
        self.updateTime = update_time
        self.updateMillisec = update_millisec
        self.bidPrice1 = bid_price1
        self.bidVolume1 = bid_volume1
        self.askPrice1 = ask_price1
        self.askVolume1 = ask_volume1
        self.bidPrice2 = bid_price2
        self.bidVolume2 = bid_volume2
        self.askPrice2 = ask_price2
        self.askVolume2 = ask_volume2
        self.bidPrice3 = bid_price3
        self.bidVolume3 = bid_volume3
        self.askPrice3 = ask_price3
        self.askVolume3 = ask_volume3
        self.bidPrice4 = bid_price4
        self.bidVolume4 = bid_volume4
        self.askPrice4 = ask_price4
        self.askVolume4 = ask_volume4
        self.bidPrice5 = bid_price5
        self.bidVolume5 = bid_volume5
        self.askPrice5 = ask_price5
        self.askVolume5 = ask_volume5
        self.bidPrice6 = bid_price6
        self.bidVolume6 = bid_volume6
        self.askPrice6 = ask_price6
        self.askVolume6 = ask_volume6
        self.prePrice = pre_price


class SimpleStringMessage(BaseBean):
    def __init__(
            self,
            data=None
    ):
        self.data = data


class TestBean(unittest.TestCase):
    def test_init_StrategyFinanceAccountDetail(self):
        return StrategyFinanceAccountDetail(account_id=u"123", frozen_cash=1, commission=1.1)

    def test_set_StrategyFinanceAccountDetail(self):
        self.test_init_StrategyFinanceAccountDetail()

    def test_init_ArrayStrategyFinanceAccountDetails(self):
        return ArrayStrategyFinanceAccountDetails([self.test_init_StrategyFinanceAccountDetail()])

    def test_from_methods(self):
        class A(BaseBean):
            def __init__(self, a=0):
                self.a = a

            def __str__(self):
                return u"A(a=%d)" % self.a

        class B(BaseBean):
            _types = {
                u"a_list": List[A]
            }

            def __init__(self, a_list=None):
                self.a_list = a_list

        # s = "{\"a\":1}"
        s = u"""{"a":1}"""
        s1 = u"{\"a_list\":[{\"a\":1},{\"a\":2}]}"
        a = A().from_json(s)
        print a
        b = B().from_json(s1)
        print b.a_list, b.a_list[0]

        print a.to_json()
        print b.to_json()



# encoding: utf-8
from __future__ import with_statement
from __future__ import absolute_import
import threading

from communication.client.protocol.interface import AbstractFramer
from communication.server import protocol
from communication.server.protocol.protocol import ProtocolMessage
from exception.exception import InvalidRequestRuntimeException
from utils.io.io import DataStream

__author__ = u'Yonka'


class LengthFramer(AbstractFramer):
    def __init__(
            self,
            io_stream,  # as python does not separate input_stream/output_stream interfaces clearly
    ):
        self.ioStream = io_stream
        self.dataStream = DataStream(io_stream, "!")
        self._r_lock = threading.Lock()
        self._w_lock = threading.Lock()

    def next_msg(self):
        with self._r_lock:
            fc = self.dataStream.read_int32()  # java readInt
            req_id = self.dataStream.read_int64()  # java readLong
            msg_type = self.dataStream.read_int32()  # java readInt
            body_length = self.dataStream.read_int32()  # java readInt
            if body_length > protocol.MAX_SIZE or body_length < 0:
                raise InvalidRequestRuntimeException(u"got unexpected body size: %d" % body_length)
            elif req_id < 0 or msg_type < 0 or fc < 0:
                raise InvalidRequestRuntimeException(u"invalid reqId or msgType or function code")
            body = self.ioStream.read(body_length)
            while len(body) != body_length:
                body += self.ioStream.read(len(body) - body_length)
            if 0 != body_length and body_length != len(body):
                raise RuntimeError(u"expected data length: %d, but only get %d" % (body_length, len(body)))
            return ProtocolMessage(fc, req_id, msg_type, body)  # bytes -> str

    def frame_msg(self, message):
        with self._w_lock:
            if message.body is not None and len(message.body.encode("utf-8")) > protocol.MAX_SIZE:
                raise ValueError(u"invalid message size")
            bs = message.to_stream_bytes('!')
            self.ioStream.write(bs)
            self.ioStream.flush()



# encoding: utf-8
from __future__ import absolute_import
import io
import struct

try:
    import errno
except ImportError:
    errno = None

__author__ = u'Yonka'

EBADF = getattr(errno, 'EBADF', 9)
EAGAIN = getattr(errno, 'EAGAIN', 11)
EWOULDBLOCK = getattr(errno, 'EWOULDBLOCK', 11)

_blocking_errnos = {EAGAIN, EWOULDBLOCK}


class timeout(OSError):
    # no doc
    def __init__(self, *args, **kwargs):  # real signature unknown
        pass

    __weakref__ = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """list of weak references to the object (if defined)"""


class error(Exception):
    """ Base class for I/O related errors. """

    def __init__(self, *args, **kwargs):  # real signature unknown
        pass

    @staticmethod  # known case of __new__
    def __new__(*args, **kwargs):  # real signature unknown
        """ Create and return a new object.  See help(type) for accurate signature. """
        pass

    def __reduce__(self, *args, **kwargs):  # real signature unknown
        pass

    def __str__(self, *args, **kwargs):  # real signature unknown
        """ Return str(self). """
        pass

    characters_written = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default

    errno = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """POSIX exception code"""

    filename = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """exception filename"""

    filename2 = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """second exception filename"""

    strerror = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """exception strerror"""

    winerror = property(lambda self: object(), lambda self, v: None, lambda self: None)  # default
    """Win32 exception code"""


class SocketIO(io.RawIOBase):
    """Raw I/O implementation for stream sockets.

    This class supports the makefile() method on sockets.  It provides
    the raw I/O interface on top of a socket object.
    """

    # One might wonder why not let FileIO do the job instead.  There are two
    # main reasons why FileIO is not adapted:
    # - it wouldn't work under Windows (where you can't used read() and
    # write() on a socket handle)
    # - it wouldn't work with socket timeouts (FileIO would ignore the
    # timeout and consider the socket non-blocking)

    # XXX More docs

    def __init__(self, sock, mode):
        if mode not in ("r", "w", "rw", "rb", "wb", "rwb"):
            raise ValueError("invalid mode: %r" % mode)
        io.RawIOBase.__init__(self)
        self._sock = sock
        if "b" not in mode:
            mode += "b"
        self._mode = mode
        self._reading = "r" in mode
        self._writing = "w" in mode
        self._timeout_occurred = False

    def readinto(self, b):
        """Read up to len(b) bytes into the writable buffer *b* and return
        the number of bytes read.  If the socket is non-blocking and no bytes
        are available, None is returned.

        If *b* is non-empty, a 0 return value indicates that the connection
        was shutdown at the other end.
        """
        self._checkClosed()
        self._checkReadable()
        if self._timeout_occurred:
            raise OSError("cannot read from timed out object")
        while True:
            try:
                return self._sock.recv_into(b)
            except timeout:
                self._timeout_occurred = True
                raise
            except error as e:
                if e.args[0] in _blocking_errnos:
                    return None
                raise

    def write(self, b):
        """Write the given bytes or bytearray object *b* to the socket
        and return the number of bytes written.  This can be less than
        len(b) if not all data could be written.  If the socket is
        non-blocking and no bytes could be written None is returned.
        """
        self._checkClosed()
        self._checkWritable()
        try:
            return self._sock.send(b)
        except error as e:
            # XXX what about EINTR?
            if e.args[0] in _blocking_errnos:
                return None
            raise

    def readable(self):
        """True if the SocketIO is open for reading.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return self._reading

    def writable(self):
        """True if the SocketIO is open for writing.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return self._writing

    def seekable(self):
        """True if the SocketIO is open for seeking.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return super(SocketIO, self).seekable()

    def fileno(self):
        """Return the file descriptor of the underlying socket.
        """
        self._checkClosed()
        return self._sock.fileno()

    @property
    def name(self):
        if not self.closed:
            return self.fileno()
        else:
            return -1

    @property
    def mode(self):
        return self._mode

    def close(self):
        """Close the SocketIO object.  This doesn't close the underlying
        socket, except if all references to it have disappeared.
        """
        if self.closed:
            return
        io.RawIOBase.close(self)
        self._sock._decref_socketios()
        self._sock = None


class DataStream(object):
    def __init__(self, base_stream, endian=""):
        if endian is None:
            endian = ""
        self._endian = endian
        self.base_stream = base_stream
        self._char_fmt = 'b'
        self._unsigned_char_fmt = 'B'
        self._bool_fmt = '?'
        self._short_fmt = endian + 'h'
        self._unsigned_short_fmt = endian + 'H'
        self._int_fmt = endian + 'i'
        self._unsigned_int_fmt = endian + 'I'
        self._long_fmt = endian + 'l'
        self._unsigned_long_fmt = endian + 'L'
        self._long_long_fmt = endian + 'q'
        self._unsigned_long_long_fmt = endian + 'Q'
        self._float_fmt = endian + 'f'
        self._double_fmt = endian + 'd'
        self._chars_fmt = endian + 's'

    def read_byte(self):
        return self.base_stream.read(1)

    def read_bytes(self, length):
        bs = self.base_stream.read(length)
        if bs is None or len(bs) == 0:
            raise IOError("read_bytes(%d), but got bs %s" % (length, bs))
        return bs

    def read_char(self):
        return self.unpack(self._char_fmt)

    def read_uchar(self):
        return self.unpack(self._unsigned_char_fmt)

    def read_bool(self):
        return self.unpack(self._bool_fmt)

    def read_int16(self):
        return self.unpack(self._short_fmt, 2)

    def read_uint16(self):
        return self.unpack(self._unsigned_short_fmt, 2)

    def read_int32(self):
        return self.unpack(self._int_fmt, 4)

    def read_uint32(self):
        return self.unpack(self._unsigned_int_fmt, 4)

    def read_int64(self):
        return self.unpack(self._long_long_fmt, 8)

    def read_uint64(self):
        return self.unpack(self._unsigned_long_long_fmt, 8)

    def read_float(self):
        return self.unpack(self._float_fmt, 4)

    def read_double(self):
        return self.unpack(self._double_fmt, 8)

    def read_string(self):
        length = self.read_uint16()
        return self.unpack(unicode(length) + u's', length)  # endian seems to do not affect this...

    def unpack(self, fmt, length=1):
        bs = self.read_bytes(length)
        res = struct.unpack(fmt, bs)[0]
        return res

    def write_bytes(self, value):
        self.base_stream.write(value)

    def write_char(self, value):
        self.pack(self._char_fmt, value)

    def write_uchar(self, value):
        self.pack(self._unsigned_char_fmt, value)

    def write_bool(self, value):
        self.pack(self._bool_fmt, value)

    def write_int16(self, value):
        self.pack(self._short_fmt, value)

    def write_uint16(self, value):
        self.pack(self._unsigned_short_fmt, value)

    def write_int32(self, value):
        self.pack(self._int_fmt, value)

    def write_uint32(self, value):
        self.pack(self._unsigned_int_fmt, value)

    def write_int64(self, value):
        self.pack(self._long_long_fmt, value)

    def write_uint64(self, value):
        self.pack(self._unsigned_long_long_fmt, value)

    def write_float(self, value):
        self.pack(self._float_fmt, value)

    def write_double(self, value):
        self.pack(self._double_fmt, value)

    def write_string(self, value):
        length = len(value)
        self.write_uint16(length)
        self.pack(unicode(length) + 's', value)

    def pack(self, fmt, data):
        return self.write_bytes(struct.pack(fmt, data))
